@@ -1,4 +1,5 @@
 #![allow(clippy::upper_case_acronyms)]
+#![allow(missing_docs)]
 
 #[macro_use]
 mod macros;
@@ -8,10 +9,8 @@ pub mod base;
 pub mod ccs;
 pub mod codec;
 pub mod deframer;
-#[allow(non_camel_case_types)]
 pub mod enums;
 pub mod fragmenter;
-#[allow(non_camel_case_types)]
 pub mod handshake;
 pub mod hsjoiner;
 pub mod message;
@@ -24,7 +23,7 @@ mod handshake_test;
 mod persist_test;
 
 #[cfg(test)]
-mod enums_test;
+pub(crate) mod enums_test;
 
 #[cfg(test)]
 mod message_test;
